#ifndef FLASH_H_
#define FLASH_H_

#define FLASH_SELECT	portc.7
#define FLASH_DATA_OUT	portb.5 // out from the POV of the PIC
#define FLASH_DATA_IN	porta.0 // in from the POV of the PIC
#define FLASH_CLOCK		porta.5

#include "common/record.h"

void flash_deleteAll();

void flash_shiftOutMSB(uint8_t val);

uint8_t flash_shiftInMSB();

void flash_deleteSector(unsigned short sector);

void flash_prepPageForWrite(unsigned short page);

void flash_readRecord(unsigned short page);

#include <system.h>
#include "flash.h"

extern bit LED;
extern unsigned char ext1;

/* NOTE: all commands are shifted in and out of the flash memory MSB first, but the user data is going to
be stored LSB first because that is the order the bus uses for communicating w/ other PICs */

// instructions for the flash mem
#define WRITE_ENABLE	0x06
#define PAGE_PROGRAM	0x02
#define SECTOR_ERASE	0xD8
#define BULK_ERASE		0xC7
#define READ			0x03

// drive chip select low and shift out an 8-bit instruction.  leaves chip select low.
void flash_beginInstruction(uint8_t instr);

// drive chip select high
void flash_endInstruction();

// shift out 8 bits of data, MSB first.  leaves clock low.
void flash_shiftOutMSB(uint8_t val);

uint8_t flash_shiftInMSB();

// shift out 8 bits of data, LSB first.  leaves clock low.
void flash_shiftOutLSB(uint8_t val);

uint8_t flash_shiftInLSB();

void flash_eraseAll();

void flash_prepPageForWrite(unsigned short page);

void flash_endPageWrite();

void flash_prepPageForRead(unsigned short page);

void flash_endPageRead();

void flash_readRecord(struct Record *rec);

#endif