#include <system.h>

#pragma DATA _CONFIG, _INTRC_OSC_NOCLKOUT & _CP_OFF & _CPD_OFF & _PWRTE_ON & _WDT_ON & _MCLRE_OFF & _BODEN_ON
#pragma CLOCK_FREQ 4000000

#define BOOSTER_SET_1 GP2
#define BOOSTER_SET_2 GP4

#define BUTTON		GP5
#define MAS_SIGNAL	GP0
#define MC2_SIGNAL	GP1

#define SIGNAL_TIMEOUT 10000
#define BUTTON_TIMEOUT 50000

void main()
{
	unsigned short periods = 0;
	unsigned short signalTimeout = 0;
	unsigned long buttonTimeout = 0;

	trisio = 0; // all pins outputs
	ansel = 0; // disable A/D
	gpio = 0; // all pins low
	
	cmcon |= 0x7; // turn off comparator
	
	// make neccessary pins inputs
	set_bit(trisio, BUTTON);
	set_bit(trisio, MAS_SIGNAL);
	set_bit(trisio, MC2_SIGNAL);
	
	// retrieve ocscal and set it
	asm
	{
		bsf _status, RP0  	; Bank 1
		call 0x3ff 	; Get the cal value
		movwf _osccal 	; Calibrate
		bcf _status, RP0 	; Bank 0
	}
	
	clear_bit(option_reg, T0CS); // TMR0 operates on internal clock
	clear_bit(option_reg, NOT_GPPU); // enable weak pull up resistors
	
	//clear_wdt();
	/*clear_bit(option_reg, PSA); // prescaler is not assigned to TMR0
	
	// set prescaler to 110 (1:128, approx every 30ms)
	set_bit(option_reg, PS2);
	clear_bit(option_reg, PS1);
	set_bit(option_reg, PS0);
	*/
	//set_bit(intcon, GIE);
	
	//set_bit(intcon, T0IE); //enable TMR0 overflow bit   
	
	//set_bit(trisio, GPIO2); // make IO pin 2 an input
	
	tmr0 = 255 - 27;
	
	while (1)
	{
		buttonTimeout = 0;
		signalTimeout = 0;
				
		while (signalTimeout < SIGNAL_TIMEOUT && buttonTimeout < BUTTON_TIMEOUT)
		{
			if (test_bit(gpio, MAS_SIGNAL) || !test_bit(gpio, MC2_SIGNAL))
				++signalTimeout;
			else
				signalTimeout = 0;
				
			if (gpio.BUTTON == 0)
				++buttonTimeout;
			else
				buttonTimeout = 0;
				
			clear_wdt();
		}
		
		/*// no, that isn't an arbitrary number... it's approximately 200ms assuming the loop takes 30us
		// this loop activates only booster set 1
		for (periods = 0; periods < 6666; ++periods)
		{
			asm
			{
				nop;
				nop;
				bsf _gpio, BOOSTER_SET_1;
				nop;
				nop;
				nop;
				nop;
				nop;
				nop;
				nop;
				nop;
				nop;
				nop;
				nop;
				nop;
				bcf _gpio, BOOSTER_SET_1;
			}
			
			clear_wdt();
		}*/
		
		// this one IS arbitrary and should be some nice long amount of time like 1sec
		// this loop activates both booster sets
		for (periods = 0; periods < 30000; ++periods)
		{
			asm
			{
				nop;
				bsf _gpio, BOOSTER_SET_1;
				bsf _gpio, BOOSTER_SET_2;
				nop;
				nop;
				nop;
				nop;
				nop;
				nop;
				nop;
				nop;
				nop;
				nop;
				nop;
				bcf _gpio, BOOSTER_SET_1;
				bcf _gpio, BOOSTER_SET_2;
			}
			
			clear_wdt();
		}
	}
}

void interrupt( void )
{
	clear_bit(intcon, T0IF);
	tmr0 = 255 - 25;
	gpio.5 ^= 1;
}
